﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Tutorial_CS_Kaufhaus_Sets
{
    internal class Program
    {
        static void Main(string[] args)
        {
            HashSet<Ware> haus1 = new HashSet<Ware>()
            {
                new Ware{Bezeichnung="Hemd",Preis=23.45},
                new Ware{Bezeichnung="Socken",Preis=5.99},
                new Ware{Bezeichnung="Grill",Preis=99.99},
                new Ware{Bezeichnung="Gartenstuhl",Preis=49.99},
                new Ware{Bezeichnung="Badetuch",Preis=12.99},
                new Ware{Bezeichnung="Kochtopf",Preis=19.89},
                new Ware{Bezeichnung="Roller",Preis=99.99},
                new Ware{Bezeichnung="Roller",Preis=99.99}
            };

            SortedSet<Ware> haus2 = new SortedSet<Ware>
            {
                new Ware{Bezeichnung = "Grill",Preis=89.99},
                new Ware{Bezeichnung="Gartenstuhl",Preis=39.89},
                new Ware{Bezeichnung="Strohhut",Preis=34.56},
                new Ware{Bezeichnung="Kochtopf",Preis=23.45 },
                new Ware{Bezeichnung="Gummistiefel",Preis=11.99}
            };

            //Ausgabe der Anzahl aller Waren
            Console.WriteLine("Anzahl: " + haus1.Count());

            Console.WriteLine("---- nur Bezeichnung -------");

            //Ausgabe aller Bezeichnungen 
            foreach(string bez in haus1.Select(w => w.Bezeichnung))
            {
                Console.WriteLine(bez);
            }

            Console.WriteLine("Feuerzeug");
            //Ausgabe der Ware mit einer bestimmten Bezeichnung 
            Ware erg = haus1.SingleOrDefault(w => w.Bezeichnung == "Feuerzeug");
            Console.WriteLine((erg == null) ? "N/A" : erg.ToString());


            Console.WriteLine("--- Preis zwischen----");
            //Ausgabe aller Waren mit preis zwischen 20 und 60
            foreach(Ware w in haus1.Where(w => w.Preis > 20 && w.Preis < 60))
            {
                Console.WriteLine(w);
            }

            Console.WriteLine("----------- Durchschnitt ---------");
            //Ausgabe Durchschnittspreis aller Waren

            Console.WriteLine("Durchschnitt: " + haus1.Average(w => w.Preis));

            Console.WriteLine("--- Sortierung preis aufsteigend ----------");
            //Sortierte Ausgabe nach Preis aufsteigend
            foreach(Ware w in haus1.OrderBy(w => w.Preis))
            {
                Console.WriteLine(w);
            }

            Console.WriteLine("-------- Ware mit max Preis ----------");
            //Ausgabe Waren mit höchstem Preis 

            double max = haus1.Max(w => w.Preis);
            foreach(Ware w in haus1.Where(w => w.Preis == max))
            {
                Console.WriteLine(w);
            }

            Console.WriteLine("-------- mehr als Durchschnitt ---------");
            //Ausgabe aller Waren, die mehr als Durchschnitt kosten 
            double avg = haus1.Average(w => w.Preis);
            foreach(Ware w in haus1.Where(w => w.Preis > avg))
            {
                Console.WriteLine(w);
            }

            Console.WriteLine("------- Günstig 3 ----------");
            //Ausgabe der drei günstigsten Waren
            foreach(Ware w in haus1.OrderBy(w => w.Preis).Take(3))
            {
                Console.WriteLine(w);
            }

            Console.WriteLine("-----  Ergänzen ---------");

            //Jede Ware um Steuer ergänzen bei 19%
            foreach(var obj in haus1.Select(w => new {w.Bezeichnung, w.Preis, Steuer = w.Preis /119 * 19}))
            {
                Console.WriteLine(obj.Bezeichnung + "|" + obj.Preis + "|" + obj.Steuer);
            }

            Console.WriteLine("*******************************************");


           

            foreach(Ware w in haus2)
            {
                Console.WriteLine(w);
            }

            Console.WriteLine("-----");

            Ware w1 = new Ware { Bezeichnung = "Schuhe", Preis = 54.99 };
            Ware w2 = new Ware { Bezeichnung = "Lamadecke", Preis = 129.99 };

            //Schuhe verglichen mit Lamadecke
            Console.WriteLine(w1.CompareTo(w2));

            Console.WriteLine("--------------");

            //Zusammenfassung - Waren plus Ausgabe in welchem Haus
            var alle = haus1.Select(w => new { Haus = "Haus1", w.Bezeichnung, w.Preis }).Union(haus2.Select(w => new { Haus = "Haus2", w.Bezeichnung, w.Preis }));

            foreach(var obj in alle)
            {
                Console.WriteLine(obj.Bezeichnung + "|" + obj.Preis);
            }

            Console.WriteLine("--- Gruppierung nach Bezeichnung ------ \n");
            //Auflistung der Waren in beiden Kaufhäusern
            foreach(var o in alle.GroupBy(obj => obj.Bezeichnung, (key,value)=>new {Bez = key, Liste = value}))
            {
                Console.WriteLine(o.Bez);
                foreach(var w in o.Liste)
                {
                    Console.WriteLine("\t" + w.Haus + "|" + w.Preis);
                }
            }

            Console.WriteLine("--- Auflistung - welche ware welches haus am günstigsten ----");

            var ausgabe = alle.GroupBy(obj => obj.Bezeichnung, (key, value) => new { Bez = key, Ware = value.First(p => p.Preis == value.Min(o => o.Preis)) });

            foreach(var obj in ausgabe)
            {
                Console.WriteLine(obj.Bez);
                Console.WriteLine("\t" + obj.Ware);
            }

            Console.WriteLine("--- Welche Waren nur in haus2 ---");

            foreach(Ware w in haus2.Except(haus1, new Vergleich()))
            {
                Console.WriteLine(w);
            }

            Console.WriteLine("--- Welche Waren in beiden Häusern --------");

            foreach(Ware w in haus1.Intersect(haus2, new Vergleich()))
            {
                Console.WriteLine(w);
            }

            Console.WriteLine("---- Welches haus hat Gummistiefel ----");
            var liste = alle.Where(obj => obj.Bezeichnung == "Gummistiefel");
            foreach(var obj in liste)
            {
                Console.WriteLine(obj.Haus + "|" + obj.Bezeichnung + "|" + obj.Preis);
            }

            Console.WriteLine("-----------------------");

            //----------- BEISPIEL 1 ----------------
            foreach(string obj in haus1.Select(w => w.Bezeichnung).Union(haus2.Select(w => w.Bezeichnung)))
            {
                Console.WriteLine(obj);
            }
        }
    }
}
